// Bine is a toolkit to assist in creating Tor clients and servers. Features:
//
// * Full support for the Tor controller API
//
// * Support for `net.Conn` and `net.Listen` style APIs
//
// * Supports statically compiled Tor to embed Tor into the binary
//
// * Supports both v2 and v3 onion services
//
// * Support for embedded control socket in Tor >= 0.3.5 (non-Windows)
//
// Users of this library will usually use the high-level tor package. See
// README at https://github.com/cretz/bine for more info.
package bine
