// Package tor is the high-level client for Tor.
//
// The Tor type is a combination of a Tor instance and a connection to it.
// Use Start to create Tor. Then Dialer or Listener can be used.
//
// Some of this code is lifted from https://github.com/yawning/bulb with thanks.
package tor
